//#outname "testmain2.java"
package com.igormaznitsa.dummyproject;

//#-
public class testmain {
//#+
//$$public class testmain2 {
public String test(){
        return "/*$mvn.project.name$*/";
    }
}