/* 
 * Copyright 2014 Igor Maznitsa (http://www.igormaznitsa.com).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.igormaznitsa.jcp.expression;

public enum ExpressionItemPriority {

  LOGICAL(0),
  COMPARISON(1),
  ARITHMETIC_ADD_SUB(2),
  ARITHMETIC_MUL_DIV_MOD(3),
  FUNCTION(5),
  VALUE(6);

  private final int priority;

  public int getPriority() {
    return priority;
  }

  private ExpressionItemPriority(final int priority) {
    this.priority = priority;
  }
}
